/*
 *  Copyright (C) 2017  Red Hat, Inc.
 *
 *  This work is licensed under the terms of the GNU GPL, version 2. See
 *  the COPYING file in the top-level directory.
 */

#include <linux/cpumask.h>
#include <linux/debugfs.h>
#include <linux/mutex.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <asm/smp.h>
#include <asm-generic/spec_ctrl.h>
#include <asm-generic/nospec-branch.h>
#include <asm-x86_64/cpufeature.h>

static DEFINE_MUTEX(spec_ctrl_mutex);

static bool noibrs_cmdline __read_mostly = false;
static bool ibp_disabled __read_mostly = false;
unsigned int ibrs_mode __read_mostly;
EXPORT_SYMBOL(ibrs_mode);

/*
 * The Intel specification for the SPEC_CTRL MSR requires that we
 * preserve any already set reserved bits at boot time (e.g. for
 * future additions that this kernel is not currently aware of).
 * We then set any additional mitigation bits that we want
 * ourselves and always use this as the base for SPEC_CTRL.
 * We also use this when handling guest entry/exit as below.
 *
 * RHEL note: We do the above to be in sync with upstream,
 * but in the RHEL case, we have both x86_spec_ctrl_base,
 * and a PER_CPU spec_ctrl_pcp to track and manage.
 *
 * RHEL note: It's actually cleaner to directly export this
 * and allow all of our assorted IBRS management code to touch
 * this directly, rather than use the upstream accessors. We
 * implement them, but we don't use those in the RHEL code.
 */

/*
 * Our boot-time value of the SPEC_CTRL MSR. We read it once so that any
 * writes to SPEC_CTRL contain whatever reserved bits have been set.
 */
u64 __read_mostly x86_spec_ctrl_base;
EXPORT_SYMBOL_GPL(x86_spec_ctrl_base);

/*
 * AMD specific MSR info for Store Bypass control.
 * x86_amd_ls_cfg_ssbd_mask is initialized in identify_boot_cpu().
 */
u64 __read_mostly x86_amd_ls_cfg_base;
u64 __read_mostly x86_amd_ls_cfg_ssbd_mask;

void spec_ctrl_save_msr(void)
{
	int cpu;
	unsigned int hival, loval;

	/*
	 * Read the SPEC_CTRL MSR to account for reserved bits which may have
	 * unknown values. AMD64_LS_CFG MSR is cached in the early AMD
	 * init code as it is not enumerated and depends on the family.
	 */
	if (rh_cpu_test_cap(X86_FEATURE_IBRS))
		rdmsrl(MSR_IA32_SPEC_CTRL, x86_spec_ctrl_base);

	/*
	 * RHEL only: update the PER_CPU spec_ctrl_pcp cached values
	 */

	loval = x86_spec_ctrl_base & 0xffffffff;
	hival = (x86_spec_ctrl_base >> 32) & 0xffffffff;

	for_each_possible_cpu(cpu) {
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.hi32, hival);
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.entry, loval);
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.exit, loval);
	}
}

/*
 * RHEL note: We implement this in RHEL, but we don't use it directly since
 * we have a lot of IBRS management code that touches SPEC_CTRL directly. It
 * turns out to be cleaner to allow it to do that (we tested both approaches).
 */
void x86_spec_ctrl_set(u64 val)
{
	if (val & ~FEATURE_ENABLE_IBRS)
		WARN_ONCE(1, "SPEC_CTRL MSR value 0x%16llx is unknown.\n", val);
	else
		wrmsrl(MSR_IA32_SPEC_CTRL, x86_spec_ctrl_base | val);
}
EXPORT_SYMBOL_GPL(x86_spec_ctrl_set);

u64 x86_spec_ctrl_get_default(void)
{
	return x86_spec_ctrl_base;
}
EXPORT_SYMBOL_GPL(x86_spec_ctrl_get_default);

static void set_spec_ctrl_pcp(bool entry, bool exit)
{
	unsigned int enabled   = read_pda(spec_ctrl_pcp.enabled);
	unsigned int entry_val = read_pda(spec_ctrl_pcp.entry);
	unsigned int exit_val  = read_pda(spec_ctrl_pcp.exit);
	int cpu;

	/*
	 * For ibrs_always, we only need to write the MSR at kernel entry
	 * to fulfill the barrier semantics for some CPUs.
	 */
	if (entry && exit)
		enabled = SPEC_CTRL_PCP_IBRS_ENTRY;
	else if (entry != exit)
		enabled = SPEC_CTRL_PCP_IBRS_ENTRY|SPEC_CTRL_PCP_IBRS_EXIT;
	else
		enabled = 0;

	if (entry)
		entry_val |= FEATURE_ENABLE_IBRS;
	else
		entry_val &= ~FEATURE_ENABLE_IBRS;

	if (exit)
		exit_val |= FEATURE_ENABLE_IBRS;
	else
		exit_val &= ~FEATURE_ENABLE_IBRS;

	for_each_possible_cpu(cpu) {
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.enabled, enabled);
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.entry, entry_val);
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.exit, exit_val);
	}
}

/*
 * The following values are written to IBRS on kernel entry/exit:
 *
 *		entry	exit
 * ibrs		   1	  0
 * ibrs_always	   1	  1
 * ibrs_user	   0	  1
 */
static void set_spec_ctrl_pcp_ibrs(void)
{
	set_spec_ctrl_pcp(true, false);
	ibrs_mode = IBRS_ENABLED;
}

static void set_spec_ctrl_pcp_ibrs_always(void)
{
	set_spec_ctrl_pcp(true, true);
	ibrs_mode = IBRS_ENABLED_ALWAYS;
}

static void set_spec_ctrl_pcp_ibrs_user(void)
{
	set_spec_ctrl_pcp(false, true);
	ibrs_mode = IBRS_ENABLED_USER;
}

void clear_spec_ctrl_pcp(void)
{
	set_spec_ctrl_pcp(false, false);
	ibrs_mode = IBRS_DISABLED;
}

static void spec_ctrl_sync_all_cpus(u32 msr_nr, u64 val)
{
	int cpu;
	struct __msr msr_val = {{ .q = val }};

	for_each_online_cpu(cpu)
		wrmsr_on_cpu(cpu, msr_nr, msr_val.l, msr_val.h);
}

static void sync_all_cpus_ibrs(bool enable)
{
	spec_ctrl_sync_all_cpus(MSR_IA32_SPEC_CTRL,
				 enable ? (x86_spec_ctrl_base | FEATURE_ENABLE_IBRS)
					: x86_spec_ctrl_base);
}

static void __sync_this_cpu_ibp(void *data)
{
	bool enable = *(bool *)data;
	u64 val;

	/* disable IBP on old CPU families */
	rdmsrl(MSR_F15H_IC_CFG, val);

	if (!enable)
		val |= MSR_F15H_IC_CFG_DIS_IND;
	else
		val &= ~MSR_F15H_IC_CFG_DIS_IND;

	wrmsrl(MSR_F15H_IC_CFG, val);
}

/* enable means IBP should be enabled in the CPU (i.e. fast) */
static void sync_all_cpus_ibp(bool enable)
{
	__sync_this_cpu_ibp(&enable);

	smp_call_function_many(&cpu_online_map, __sync_this_cpu_ibp,
				&enable, 0, 1);
}

static void spec_ctrl_disable_all(void)
{
	int cpu;

	for_each_possible_cpu(cpu)
		WRITE_ONCE(cpu_pda(cpu)->spec_ctrl_pcp.exit, 0);
	spectre_v2_retpoline_reset();
}

static int __init noibrs(char *str)
{
	noibrs_cmdline = true;

	return 0;
}
early_param("noibrs", noibrs);

static int __init noibpb(char *str)
{
	/* deprecated */
	return 0;
}
early_param("noibpb", noibpb);

bool spec_ctrl_force_enable_ibrs(void)
{
	if (cpu_has_spec_ctrl()) {
		set_spec_ctrl_pcp_ibrs();
		spectre_v2_set_mitigation(SPECTRE_V2_IBRS);
		return true;
	}
	return false;
}

bool spec_ctrl_cond_enable_ibrs(bool full_retp)
{
	if (cpu_has_spec_ctrl() && !full_retp && !noibrs_cmdline) {
		set_spec_ctrl_pcp_ibrs();
		spectre_v2_set_mitigation(SPECTRE_V2_IBRS);
		return true;
	}
	return false;
}

bool spec_ctrl_enable_ibrs_always(void)
{
	if (cpu_has_spec_ctrl()) {
		set_spec_ctrl_pcp_ibrs_always();
		spectre_v2_set_mitigation(SPECTRE_V2_IBRS_ALWAYS);
		return true;
	}
	return false;
}

void spec_ctrl_enable_retpoline_ibrs_user(void)
{
	if (!cpu_has_spec_ctrl() || !spectre_v2_has_full_retpoline())
		return;

	set_spec_ctrl_pcp_ibrs_user();
	spectre_v2_set_mitigation(SPECTRE_V2_RETPOLINE_IBRS_USER);
	return;
}

bool spec_ctrl_force_enable_ibp_disabled(void)
{
	/*
	 * Some AMD CPUs don't need IBPB or IBRS CPUID bits, because
	 * they can just disable indirect branch predictor
	 * support (MSR 0xc0011021[14]).
	 */
	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE)) {
		ibp_disabled = true;
		spectre_v2_set_mitigation(SPECTRE_V2_IBP_DISABLED);
		return true;
	}
	ibp_disabled = false;
	return false;
}

bool spec_ctrl_cond_enable_ibp_disabled(void)
{
	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE) && !noibrs_cmdline) {
		ibp_disabled = true;
		spectre_v2_set_mitigation(SPECTRE_V2_IBP_DISABLED);
		return true;
	}
	ibp_disabled = false;
	return false;
}

static void spec_ctrl_print_features(void)
{
	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE)) {
		printk_once(KERN_INFO "FEATURE SPEC_CTRL Present (Implicit)\n");
		printk_once(KERN_INFO "FEATURE IBPB_SUPPORT Present (Implicit)\n");
		return;
	}

	if (cpu_has_spec_ctrl())
		printk_once(KERN_INFO "FEATURE SPEC_CTRL Present\n");
	else
		printk_once(KERN_INFO "FEATURE SPEC_CTRL Not Present\n");

	if (rh_cpu_test_cap(X86_FEATURE_IBPB))
		printk_once(KERN_INFO "FEATURE IBPB_SUPPORT Present\n");
	else
		printk_once(KERN_INFO "FEATURE IBPB_SUPPORT Not Present\n");
}

void spec_ctrl_cpu_init(void)
{
	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE)) {
		bool enabled = !ibp_disabled;
		__sync_this_cpu_ibp(&enabled);
		return;
	}

	if (ibrs_mode == IBRS_ENABLED_ALWAYS)
		wrmsrl(MSR_IA32_SPEC_CTRL,
		       x86_spec_ctrl_base | FEATURE_ENABLE_IBRS);
}

static void spec_ctrl_reinit_all_cpus(void)
{
	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE)) {
		sync_all_cpus_ibp(!ibrs_mode);
		return;
	}

	if (ibrs_mode == IBRS_ENABLED_ALWAYS)
		sync_all_cpus_ibrs(true);
	else if (ibrs_mode == IBRS_DISABLED)
		sync_all_cpus_ibrs(false);
}



static ssize_t __enabled_read(struct file *file, char __user *user_buf,
			      size_t count, loff_t *ppos, unsigned int *field)
{
	char buf[32];
	unsigned int len;

	len = sprintf(buf, "%d\n", READ_ONCE(*field));
	return simple_read_from_buffer(user_buf, count, ppos, buf, len);
}

static ssize_t ibrs_enabled_read(struct file *file, char __user *user_buf,
				 size_t count, loff_t *ppos)
{
	unsigned int enabled = ibrs_mode;

	if (ibp_disabled)
		enabled = IBRS_ENABLED_ALWAYS;

	return __enabled_read(file, user_buf, count, ppos, &enabled);
}

static ssize_t ibrs_enabled_write(struct file *file,
				  const char __user *user_buf,
				  size_t count, loff_t *ppos)
{
	char buf[32];
	ssize_t len;
	unsigned int enable;

	len = min(count, sizeof(buf) - 1);
	if (copy_from_user(buf, user_buf, len))
		return -EFAULT;

	buf[len] = '\0';
	enable = (unsigned int)simple_strtoul(buf, NULL, 0);
	if (enable > IBRS_MAX)
		return -EINVAL;

	mutex_lock(&spec_ctrl_mutex);
	if ((!ibp_disabled && enable == ibrs_mode) ||
	    (ibp_disabled && enable == IBRS_ENABLED_ALWAYS))
		goto out_unlock;

	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE)) {
		if (enable == IBRS_ENABLED || enable == IBRS_ENABLED_USER) {
			count = -EINVAL;
			goto out_unlock;
		}

		if (enable == IBRS_DISABLED) {
			sync_all_cpus_ibp(true);
			ibp_disabled = false;
			spectre_v2_retpoline_reset();
		} else {
			WARN_ON(enable != IBRS_ENABLED_ALWAYS);
			sync_all_cpus_ibp(false);
			ibp_disabled = true;
			spectre_v2_set_mitigation(SPECTRE_V2_IBP_DISABLED);
		}
		goto out_unlock;
	}

	if (!cpu_has_spec_ctrl()) {
		count = -ENODEV;
		goto out_unlock;
	}

	if (enable == IBRS_DISABLED) {
		clear_spec_ctrl_pcp();
		sync_all_cpus_ibrs(false);
		spectre_v2_retpoline_reset();
	} else if (enable == IBRS_ENABLED) {
		set_spec_ctrl_pcp_ibrs();
		spectre_v2_set_mitigation(SPECTRE_V2_IBRS);
	} else if (enable == IBRS_ENABLED_ALWAYS) {
		set_spec_ctrl_pcp_ibrs_always();
		sync_all_cpus_ibrs(true);
		spectre_v2_set_mitigation(SPECTRE_V2_IBRS_ALWAYS);
	} else {
		WARN_ON(enable != IBRS_ENABLED_USER);
		if (!spectre_v2_has_full_retpoline()) {
			count = -ENODEV;
			goto out_unlock;
		}
		set_spec_ctrl_pcp_ibrs_user();
		spectre_v2_set_mitigation(SPECTRE_V2_RETPOLINE_IBRS_USER);
	}

out_unlock:
	mutex_unlock(&spec_ctrl_mutex);
	return count;
}

static const struct file_operations fops_ibrs_enabled = {
	.read = ibrs_enabled_read,
	.write = ibrs_enabled_write,
	.llseek = default_llseek,
};

static ssize_t ibpb_enabled_read(struct file *file, char __user *user_buf,
				 size_t count, loff_t *ppos)
{
	unsigned int enabled = ibpb_enabled();

	if (ibp_disabled)
		enabled = 1;

	return __enabled_read(file, user_buf, count, ppos, &enabled);
}

static const struct file_operations fops_ibpb_enabled = {
	.read = ibpb_enabled_read,
	.llseek = default_llseek,
};

static ssize_t retp_enabled_read(struct file *file, char __user *user_buf,
				 size_t count, loff_t *ppos)
{
	unsigned int enabled = !!rh_cpu_test_cap(X86_FEATURE_RETPOLINE);

	return __enabled_read(file, user_buf, count, ppos, &enabled);
}

static const struct file_operations fops_retp_enabled = {
	.read = retp_enabled_read,
	.llseek = default_llseek,
};

static int __init debugfs_spec_ctrl(void)
{
	debugfs_create_file("ibrs_enabled", S_IRUSR | S_IWUSR,
			    arch_debugfs_dir, NULL, &fops_ibrs_enabled);
	debugfs_create_file("ibpb_enabled", S_IRUSR,
			    arch_debugfs_dir, NULL, &fops_ibpb_enabled);
	debugfs_create_file("retp_enabled", S_IRUSR,
			    arch_debugfs_dir, NULL, &fops_retp_enabled);

	return 0;
}
late_initcall(debugfs_spec_ctrl);

void spec_ctrl_rescan_cpuid(void)
{
	enum spectre_v2_mitigation old_mode;
	bool old_ibrs, old_ibpb, old_ssbd;
	bool ssbd_changed;
	int cpu;

	if (rh_cpu_test_cap(X86_FEATURE_IBP_DISABLE))
		return;

	mutex_lock(&spec_ctrl_mutex);
	if (boot_cpu_data.x86_vendor == X86_VENDOR_INTEL ||
	    boot_cpu_data.x86_vendor == X86_VENDOR_AMD) {

		old_ibrs = rh_cpu_test_cap(X86_FEATURE_IBRS);
		old_ibpb = rh_cpu_test_cap(X86_FEATURE_IBPB);
		old_ssbd  = rh_cpu_test_cap(X86_FEATURE_SSBD);
		old_mode = spectre_v2_get_mitigation();

		/* detect spec ctrl related cpuid additions */
		get_scattered_cpuid_extra(&boot_cpu_data);

		/*
  		 * redo init_speculation_control logic as X86_FEATURE_SPEC_CTRL
  		 * and X86_FEATURE_INTEL_STIBP might have been set
  		 */
	        if (rh_cpu_test_cap(X86_FEATURE_SPEC_CTRL)) {
	               	rh_cpu_set_cap(X86_FEATURE_IBRS);
                	rh_cpu_set_cap(X86_FEATURE_IBPB);
        	}
        	if (rh_cpu_test_cap(X86_FEATURE_INTEL_STIBP)) {
                	rh_cpu_set_cap(X86_FEATURE_STIBP);
        	}

		if (rh_cpu_test_cap(X86_FEATURE_SPEC_CTRL) &&
			rh_cpu_test_cap(X86_FEATURE_INTEL_STIBP))
			rh_cpu_set_cap(X86_FEATURE_SSBD);

		/* if there were no spec ctrl related changes, we're done */
		ssbd_changed = (old_ssbd != rh_cpu_test_cap(X86_FEATURE_SSBD));
		if (old_ibrs == rh_cpu_test_cap(X86_FEATURE_IBRS) &&
		    old_ibpb == rh_cpu_test_cap(X86_FEATURE_IBPB) && !ssbd_changed)
			goto done;

		/*
		 * we just have the global feature, so no propagation to other
		 * cpus
		 */

		/* print the changed IBRS/IBPB features */
		spec_ctrl_print_features();

		if (ssbd_changed) {
			/*
			 * Redo speculation store bypass setup.
			 */
			ssb_select_mitigation();
			if (x86_spec_ctrl_base) {
				/*
				 * Need to propagate the new baseline to all
				 * the percpu spec_ctrl structures. The
				 * spectre v2 re-initialization below will
				 * reset the right percpu values.
				 */
				/*
				 * This is the point where x86_spec_ctrl_base
				 * is first written. There is at least one
				 * microcode version/cpu combination where the
				 * vulnerability is reported,
				 * X86_FEATURE_INTEL_STIBP is set, but setting
				 * the enable bit causes
				 * an exception (setting other bits in the
				 * same register succeed, though). Detect that
				 * condition here and if we get the exception
				 * print a message and reset SSBD related
				 * X86_FEATURE* bits and reset the ssb
				 * mitigation
				 */
				if (wrmsr_safe(MSR_IA32_SPEC_CTRL,
					       (u32)x86_spec_ctrl_base,
					       (u32)(x86_spec_ctrl_base >> 32))
				    < 0) {
					printk("broken microcode detected, "
					       " disabling SSBD related "
					       "features\n");
					rh_cpu_clear_cap(X86_FEATURE_SSBD);
					rh_cpu_clear_cap(X86_FEATURE_INTEL_STIBP);
					rh_cpu_clear_cap(X86_FEATURE_STIBP);
					x86_spec_ctrl_base &=
						~FEATURE_ENABLE_SSBD;
					ssb_select_mitigation();
				} else {
					spec_ctrl_save_msr();
					sync_all_cpus_ibrs(false);
				}
			}
		}

		/*
		 * Re-execute the v2 mitigation logic based on any new CPU
		 * features. Note that any debugfs-based changes the user may
		 * have made will be overwritten, because new features are now
		 * available, so any previous changes may no longer be relevant.
		 * Go back to the defaults unless they're overridden
		 * by the cmdline.
		 */
		spec_ctrl_disable_all();
		__spectre_v2_select_mitigation();
		spec_ctrl_reinit_all_cpus();

		/* print any mitigation changes */
		if (old_mode != spectre_v2_get_mitigation())
			spectre_v2_print_mitigation();
	}
done:
	mutex_unlock(&spec_ctrl_mutex);
}
EXPORT_SYMBOL_GPL(spec_ctrl_rescan_cpuid);
